@@ -1,16 +1,30 @@
-# This library is in Development and bugs can be expected. If you encounter any bugs, want to give feedback, or would like to contribute, join our Discord server.
+# This library is in development and bugs can be expected. If you encounter any bugs, want to give feedback, or would like to contribute, join our Discord server.
 # https://discord.gg/TUUbzTa3B7
 
 """
-Copyright 2021 George Luca
+Copyright (c) 2021 George Luca
 
-Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
+Permission is hereby granted, free of charge, to any person obtaining a copy
+of this software and associated documentation files (the "Software"), to deal
+in the Software without restriction, including without limitation the rights
+to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+copies of the Software, and to permit persons to whom the Software is
+furnished to do so, subject to the following conditions:
 
-The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
+The above copyright notice and this permission notice shall be included in all
+copies or substantial portions of the Software.
 
-THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+SOFTWARE.
 """
 
+# Ignore the fact that I used text where there should be an integer in the SQL. I made this when I first started and there is not really a point in changing it now.
+
 import os
 import discord
 import datetime
@@ -18,9 +32,11 @@ import aiosqlite
 
 from discord.ext import commands
 
+
 RED = 0xD40C00
 BLUE = 0x0000FF
 GREEN = 0x32C12C
+version = "4.1.6"
 
 print("""
 ===========================================================================================
@@ -104,7 +120,10 @@ class admin:
         await ctx.message.delete()
         purged = await ctx.channel.purge(limit=amount ,after=limit)
         purged = len(purged)
-        embed = discord.Embed(color = GREEN, description = f"**Cleared {purged} messages.**")
+        if purged != 1:
+            embed = discord.Embed(color = GREEN, description = f"**Cleared {purged} messages.**")
+        else:
+            embed = discord.Embed(color = GREEN, description = f"**Cleared {purged} message.**")
         await ctx.send(embed=embed, delete_after=5)
         return purged
 
@@ -155,11 +174,12 @@ class admin:
             user = ban_entry.user
             if (user.name, user.discriminator) == (member_name, member_discriminator):
                 await ctx.guild.unban(user)
-                embed = discord.Embed(color = RED, description = f"**Unbanned {member.name}#{member.discriminator}.**")
+                embed = discord.Embed(color = RED, description = f"**Unbanned {member}.**")
                 await ctx.send(embed=embed, delete_after=7)
             else:
-                embed = discord.Embed(color = RED, description = f"**{member.name}#{member.discriminator} is not banned.**")
+                embed = discord.Embed(color = RED, description = f"**{member} is not banned.**")
                 await ctx.send(embed=embed, delete_after=5)
+                break
 class curse:
 
     async def add_banned_word(ctx, word, dir):
@@ -197,7 +217,7 @@ class curse:
                 await db.execute("INSERT INTO curses (curse,guild) VALUES (?,?)", (x,guildid))
             await db.commit()
         await db.close()
-        embed = discord.Embed(color = GREEN, description = "**Added to the list.**")
+        embed = discord.Embed(color = GREEN, description = "**Added word(s) the list.**")
         await ctx.send(embed=embed, delete_after=7)
     async def remove_banned_word(ctx, word, dir):
             db = await aiosqlite.connect("curse.db")
@@ -226,7 +246,7 @@ class curse:
                     await db.execute("DELETE FROM curses WHERE curse = ? and guild = ?", (x,guildid))
                     await db.commit()
                 await db.close()
-                embed = discord.Embed(color = GREEN, description = "**Removed from the list.**")
+                embed = discord.Embed(color = GREEN, description = "**Removed word(s) from the list.**")
                 await ctx.send(embed=embed, delete_after=5)
             except:
                 embed = discord.Embed(color = RED, description = "**A list has not been created yet.**")
@@ -400,7 +420,7 @@ class warnings:
             await ctx.send(embed=embed, delete_after=5)
             return
         os.chdir(dir)
-        reason = str(reason)
+        reason_str = str(reason)
         guildid = str(ctx.guild.id)
         user = member
         member = str(member.id)
@@ -412,10 +432,10 @@ class warnings:
         reason TEXT
         )""")
         await db.commit()
-        await db.execute("INSERT INTO warnings (member_id,guild,reason) VALUES (?,?,?)", (member,guildid,reason))
+        await db.execute("INSERT INTO warnings (member_id,guild,reason) VALUES (?,?,?)", (member,guildid,reason_str))
         await db.commit()
         await db.close()
-        if reason == None:
+        if reason is None:
             embed = discord.Embed(color = GREEN, description = f"**Warned {user.name}#{user.discriminator}.**")
             await ctx.send(embed=embed, delete_after=7)
         else:
@@ -455,10 +475,14 @@ class warnings:
         number = str(number)
         number = number.lower()
         db = await aiosqlite.connect("warnings.db")
-        async with db.execute("SELECT reason FROM warnings WHERE guild = ? and member_id = ?", (guild,member)) as cursor:
-            count = 0
-            async for entry in cursor:
-                count += 1
+        try:
+            async with db.execute("SELECT reason FROM warnings WHERE guild = ? and member_id = ?", (guild,member)) as cursor:
+                count = 0
+                async for entry in cursor:
+                    count += 1
+        except:
+            embed = discord.Embed(color = RED, description = f"**{user.name}#{user.discriminator} has no warnings.**")
+            await ctx.send(embed=embed, delete_after=5)
         if count < 1:
             embed = discord.Embed(color = RED, description = f"**{user.name}#{user.discriminator} has no warnings.**")
             await ctx.send(embed=embed, delete_after=5)
@@ -503,7 +527,7 @@ class warnings:
                         await db.execute("DELETE FROM warnings WHERE id = ?", (dict[str(number)],))
                         await db.commit()
                         await db.close()
-                        embed = discord.Embed(color = GREEN, description = f"**Cleared {user.name}#{user.discriminator}'s' #{number} warning.**")
+                        embed = discord.Embed(color = GREEN, description = f"**Cleared {user.name}#{user.discriminator}'s #{number} warning.**")
                         await ctx.send(embed=embed, delete_after=7)
                 except:
                     if number == "all":
@@ -538,10 +562,13 @@ class warnings:
                         time DATETIME
                         )""")
                         await db.commit()
-                        async with db.execute("SELECT reason FROM warnings WHERE guild = ? and member_id = ?", (guild,memberid)) as cursor:
-                            warnings_number = 0
-                            async for entry in cursor:
-                                warnings_number += 1
+                        try:
+                            async with db.execute("SELECT reason FROM warnings WHERE guild = ? and member_id = ?", (guild,memberid)) as cursor:
+                                warnings_number = 0
+                                async for _ in cursor:
+                                    warnings_number += 1
+                        except:
+                            return
                         if warnings_number == 1:
                             warnings_number_str = one
                             message = "You received your first warning."
@@ -732,15 +759,25 @@ class warnings:
             try:
                 async with db.execute("SELECT guild,member,time FROM tempmute") as cursor:
                     async for entry in cursor:
-                        guild,member,time_str = entry
+                        guild_id,member_id,time_str = entry
                         try:
-                            guild = bot.get_guild(int(guild))
+                            guild = bot.get_guild(int(guild_id))
+                            if not isinstance(guild, discord.Guild):
+                                await db.execute("DELETE FROM tempmute WHERE guild = ?", (str(guild_id),))
+                                await db.commit()
+                                continue
                             member = guild.get_member(int(member))
+                            if not isinstance(member, discord.Member):
+                                await db.execute("DELETE FROM tempmute WHERE guild = ? and member = ?", (str(guild_id),str(member_id)))
+                                await db.commit()
+                                continue
                             time = datetime.datetime.fromisoformat(time_str)
-                            role_add = await db.execute("SELECT role_add FROM mute_roles_info WHERE guild = ?", (str(guild.id),))
-                            role_add = await role_add.fetchone()
-                            role_remove = await db.execute("SELECT role_remove FROM mute_roles_info WHERE guild = ?", (str(guild.id),))
-                            role_remove = await role_remove.fetchone()
+                            role_add_cursor = await db.execute("SELECT role_add FROM mute_roles_info WHERE guild = ?", (str(guild.id),))
+                            role_add = await role_add_cursor.fetchone()
+                            await role_add_cursor.close()
+                            role_remove_cursor = await db.execute("SELECT role_remove FROM mute_roles_info WHERE guild = ?", (str(guild.id),))
+                            role_remove = await role_remove_cursor.fetchone()
+                            await role_remove_cursor.close()
                             if datetime.datetime.now() >= time:
                                 if role_remove != "None" and role_remove != "none":
                                     try:
@@ -755,7 +792,7 @@ class warnings:
                                 await db.commit()
                                 await mute_on_join.mute_remove(guild,member,dir)
                         except:
-                            pass
+                            continue
             except:
                 pass
     
@@ -765,15 +802,22 @@ class warnings:
             try:
                 async with db.execute("SELECT guild,member,time FROM tempban") as cursor:
                     async for entry in cursor:
-                        guild,member,time_str = entry
+                        guild_id,member,time_str = entry
                         try:
-                            guild = bot.get_guild(int(guild))
-                            member = guild.get_member(int(member))
+                            guild = bot.get_guild(int(guild_id))
+                            if not isinstance(guild, discord.Guild):
+                                await db.execute("DELETE FROM tempban WHERE guild = ?", (str(guild_id),))
+                                await db.commit()
+                                continue
                             time = datetime.datetime.fromisoformat(time_str)
                             if datetime.datetime.now() >= time:
-                                await db.execute("DELETE FROM tempban WHERE guild = ? and member = ? and time = ?", (str(guild.id), str(member.id), time_str))
+                                await db.execute("DELETE FROM tempban WHERE guild = ? and member = ? and time = ?", (str(guild.id), str(member), time_str))
                                 await db.commit()
-                                await guild.unban(discord.Object(id=member.id))
+                                try:
+                                    await guild.unban(discord.Object(id=int(member)))
+                                except:
+                                    await db.execute("DELETE FROM tempban WHERE guild = ? and member = ?", (str(guild.id), str(member)))
+                                    await db.commit()
                         except:
                             pass
             except:
@@ -837,14 +881,16 @@ class rr:
         if payload.member.bot:
             return
         guild = client.get_guild(payload.guild_id)
-        db = await aiosqlite.connect("rr.db")
-        async with db.execute("SELECT emoji,role FROM rr WHERE guild = ? and msg_id = ?", (str(guild.id),str(payload.message_id))) as cursor:
-            async for entry in cursor:
-                emoji,role = entry
-                role = guild.get_role(int(role))
-                if str(payload.emoji) == emoji:
-                    await payload.member.add_roles(role)
-        await db.close()
+        async with aiosqlite.connect("rr.db") as db:
+            try:
+                async with db.execute("SELECT emoji,role FROM rr WHERE guild = ? and msg_id = ?", (str(guild.id),str(payload.message_id))) as cursor:
+                    async for entry in cursor:
+                        emoji,role = entry
+                        role = guild.get_role(int(role))
+                        if str(payload.emoji) == emoji:
+                            await payload.member.add_roles(role)
+            except:
+                pass
 
     async def remove(payload, dir, client):
         os.chdir(dir)
@@ -852,96 +898,106 @@ class rr:
         member = guild.get_member(payload.user_id)
         if member.bot:
             return
-        db = await aiosqlite.connect("rr.db")
-        async with db.execute("SELECT emoji,role FROM rr WHERE guild = ? and msg_id = ?", (str(guild.id),str(payload.message_id))) as cursor:
-            async for entry in cursor:
-                emoji,role = entry
-                role = guild.get_role(int(role))
-                if str(payload.emoji) == emoji:
-                    await member.remove_roles(role)
-        await db.close()
+        async with aiosqlite.connect("rr.db") as db:
+            try:
+                async with db.execute("SELECT emoji,role FROM rr WHERE guild = ? and msg_id = ?", (str(guild.id),str(payload.message_id))) as cursor:
+                    async for entry in cursor:
+                        emoji,role = entry
+                        role = guild.get_role(int(role))
+                        if str(payload.emoji) == emoji:
+                            await member.remove_roles(role)
+            except:
+                pass
 
     async def clear_all(ctx, dir):
         os.chdir(dir)
         guild = str(ctx.guild.id)
-        db = await aiosqlite.connect("rr.db")
-        await db.execute("DELETE FROM rr WHERE guild = ?", (guild,))
-        await db.commit()
-        await db.close()
-        embed = discord.Embed(color = GREEN, description = "**Deleted all reaction role info for this server.**")
-        await ctx.send(embed=embed, delete_after=7)
+        async with aiosqlite.connect("rr.db") as db:
+            try:
+                await db.execute("DELETE FROM rr WHERE guild = ?", (guild,))
+                await db.commit()
+            except:
+                return
+            embed = discord.Embed(color = GREEN, description = "**Deleted all reaction role info for this server.**")
+            await ctx.send(embed=embed, delete_after=7)
         
     async def clear_one(ctx, dir, message_id):
         os.chdir(dir)
         guild = str(ctx.guild.id)
         message_id = str(message_id)
-        db = await aiosqlite.connect("rr.db")
-        message_id = message_id.replace(" ", "")
-        message_id = message_id.split(",")
-        for x in message_id:
-            await db.execute("DELETE FROM rr WHERE guild = ? and msg_id = ?", (guild,x))
-        await db.commit()
-        await db.close()
-        message_id = ", ".join(message_id)
-        embed = discord.Embed(color = GREEN, description = f"**Delted reaction role info for {message_id}**")
-        await ctx.send(embed=embed, delete_after=7)
+        async with aiosqlite.connect("rr.db") as db:
+            message_id = message_id.replace(" ", "")
+            message_id = message_id.split(",")
+            for x in message_id:
+                try:
+                    await db.execute("DELETE FROM rr WHERE guild = ? and msg_id = ?", (guild,x))
+                except:
+                    break
+            await db.commit()
+            message_id = ", ".join(message_id)
+            embed = discord.Embed(color = GREEN, description = f"**Delted all reaction role info with message ID(s): {message_id}**")
+            await ctx.send(embed=embed, delete_after=7)
 
     async def clear_rr(message, dir):
         os.chdir(dir)
         guild = str(message.guild.id)
         id = str(message.id)
-        db = await aiosqlite.connect("rr.db")
-        async with db.execute("SELECT msg_id FROM rr WHERE guild = ?", (guild,)) as cursor:
-            async for entry in cursor:
-                msg_id = entry[0]
-                if msg_id == id:
-                    await db.execute("DELETE FROM rr WHERE msg_id = ? and guild = ?", (msg_id,guild))
-                    await db.commit()
-                    return
-        await db.close()
+        async with aiosqlite.connect("rr.db") as db:
+            try:
+                async with db.execute("SELECT msg_id FROM rr WHERE guild = ?", (guild,)) as cursor:
+                    async for entry in cursor:
+                        msg_id = entry[0]
+                        if msg_id == id:
+                            await db.execute("DELETE FROM rr WHERE msg_id = ? and guild = ?", (msg_id,guild))
+                            await db.commit()
+                            return
+            except:
+                pass
         
     async def display(ctx, dir):
         limit = "limit"
         os.chdir(dir)
         guild = str(ctx.guild.id)
-        db = await aiosqlite.connect("rr.db")
-        embed = discord.Embed(title="Reaction Roles", color = BLUE)
-        async with db.execute("SELECT msg_id FROM rr  WHERE guild = ? GROUP BY msg_id", (guild,)) as cursor:
-            number = 0
-            async for entry in cursor:
-                number += 1
-                async with db.execute("SELECT role,emoji,channel,msg_id FROM rr WHERE guild = ? and msg_id = ?", (guild,entry[0])) as f:
-                    msg = ""
-                    msg_limit = ""
-                    async for entry in f:
-                        role,emoji,channel,msg_id = entry
-                        channel = ctx.guild.get_channel(int(channel))
-                        role = ctx.guild.get_role(int(role))
-                        try:
-                            role = role.mention
-                        except:
-                            role = "@deleted-role"
-                        try:
-                            channel = channel.mention
-                        except:
-                            channel = "#deleted-channel"
-                        msg += f"Emoji: {emoji} Role: {role}\n"
-                    msg_limit += f"Channel: {channel} Message ID: {msg_id}\n"
-                    msg += f"Channel: {channel} Message ID: {msg_id}\n"
-                if len(msg) > 1010:
-                    embed.add_field(name=f"Reaction Role #{number}", inline=False, value=msg_limit)
-                    limit = "True"
+        async with aiosqlite.connect("rr.db") as db:
+            embed = discord.Embed(title="Reaction Roles", color = BLUE)
+            try:
+                async with db.execute("SELECT msg_id FROM rr  WHERE guild = ? GROUP BY msg_id", (guild,)) as cursor:
+                    number = 0
+                    async for entry in cursor:
+                        number += 1
+                        async with db.execute("SELECT role,emoji,channel,msg_id FROM rr WHERE guild = ? and msg_id = ?", (guild,entry[0])) as f:
+                            msg = ""
+                            msg_limit = ""
+                            async for entry in f:
+                                role,emoji,channel,msg_id = entry
+                                channel = ctx.guild.get_channel(int(channel))
+                                role = ctx.guild.get_role(int(role))
+                                try:
+                                    role = role.mention
+                                except:
+                                    role = "@deleted-role"
+                                try:
+                                    channel = channel.mention
+                                except:
+                                    channel = "#deleted-channel"
+                                msg += f"Emoji: {emoji} Role: {role}\n"
+                            msg_limit += f"Channel: {channel} Message ID: {msg_id}\n"
+                            msg += f"Channel: {channel} Message ID: {msg_id}\n"
+                        if len(msg) > 1010:
+                            embed.add_field(name=f"Reaction Role #{number}", inline=False, value=msg_limit)
+                            limit = "True"
+                        else:
+                            embed.add_field(name=f"Reaction Role #{number}", inline=False, value=msg)
+                if number > 0:
+                    if limit == "limit":
+                        embed.set_footer(text=f"Total Reaction Roles | {number}")
+                    else:
+                        embed.set_footer(text=f"Total Reaction Roles | {number}")
+                        limit_embed = discord.Embed(color = RED, description = "One of your reaction roles went over the Discord limit. It will still work perfectly but only essential data will be displayed in this command to save space.")
+                        await ctx.reply(embed=limit_embed, delete_after=7)
+                    await ctx.send(embed=embed)
                 else:
-                    embed.add_field(name=f"Reaction Role #{number}", inline=False, value=msg)
-        if number > 0:
-            if limit == "limit":
-                embed.set_footer(text=f"Total Reaction Roles | {number}")
-            else:
-                embed.set_footer(text=f"Total Reaction Roles | {number}")
-                limit_embed = discord.Embed(color = RED, description = "One of your reaction roles went over the Discord limit. It will still work perfectly but only essential data will be displayed in this command to save space.")
-                await ctx.reply(embed=limit_embed, delete_after=7)
-            await ctx.send(embed=embed)
-        else:
-            embed = discord.Embed(color = RED, description = "**There are no reaction roles in this server.**")
-            await ctx.send(embed=embed, delete_after=5)
-        await db.close()
+                    embed = discord.Embed(color = RED, description = "**There are no reaction roles in this server.**")
+                    await ctx.send(embed=embed, delete_after=5)
+            except:
+                pass
